@@ -1,7 +1,6 @@
 #!/usr/bin/perl
 
-use strict;
-use warnings;
+use v5.36;
 
 use IO::Async::Loop::Epoll;
 
